package main

import (
	"errors"
	"fmt"
	"github.com/boombuler/barcode"
	"github.com/boombuler/barcode/code128"
	"github.com/boombuler/barcode/ean"
	"github.com/boombuler/barcode/pdf417"
	"github.com/boombuler/barcode/qr"
	"github.com/fatih/color"
	"github.com/jessevdk/go-flags"
	"image/png"

	//"image/png"
	"os"
)

const (
	exitOK    = 0  /* successful termination */
	exitUsage = 64 /* command line usage error */
	exitIOErr = 74 /* input/output error */
)

var parser *flags.Parser

func checkArgs(args []string) {
	if len(args) != 1 || args[0] == "" {
		printErrorAndExit(errors.New("the required `barcode' was not specified"), exitUsage)
	}
}

func writeBarcode(barcode barcode.Barcode) {

	file, err := os.Create(opts.Filename)
	if err != nil {
		printErrorAndExit(err, exitIOErr)
	}
	defer file.Close()

	err = png.Encode(file, barcode)
	if err != nil {
		printErrorAndExit(err, exitIOErr)
	}

}

type dimmension struct {
	Width  int `long:"width" short:"w" description:"Width"`
	Height int `long:"height" short:"g" description:"Height"`
}

// **********EANCOMMAND*****************

type EanCommand struct {
	dimmension
}

var eanCommand EanCommand

func (x *EanCommand) Execute(args []string) error {
	checkArgs(args)

	code, err := ean.Encode(args[0])
	if err != nil {
		printErrorAndExit(err, exitUsage)
	}

	writeBarcode(scaleBarcode(code, x.Width, x.Height))
	return nil
}

// **********QRCOMMAND*****************

type QrCommand struct {
	dimmension
}

var qrCommand QrCommand

func (x *QrCommand) Execute(args []string) error {
	checkArgs(args)

	code, err := qr.Encode(args[0], qr.M, qr.Auto)
	if err != nil {
		printErrorAndExit(err, exitUsage)
	}

	writeBarcode(scaleBarcode(code, x.Width, x.Height))
	return nil
}

// **********CODE128COMMAND*****************

type Code128Command struct {
	dimmension
}

var code128Command Code128Command

func (x *Code128Command) Execute(args []string) error {
	checkArgs(args)

	code, err := code128.Encode(args[0])
	if err != nil {
		printErrorAndExit(err, exitUsage)
	}

	writeBarcode(scaleBarcode(code, x.Width, x.Height))
	return nil
}

// **********CODE128COMMAND*****************

type PDF417Command struct {
	dimmension
	SecurityLevel byte `long:"security-level" short:"s" description:"securityLevel should be between 0 and 8. The higher the number, the more additional error-correction codes are added."`
}

var pdf417Command PDF417Command

func (x *PDF417Command) Execute(args []string) error {
	checkArgs(args)

	code, err := pdf417.Encode(args[0], x.SecurityLevel)
	if err != nil {
		printErrorAndExit(err, exitUsage)
	}

	writeBarcode(scaleBarcode(code, x.Width, x.Height))
	return nil
}

// ***************************

var opts struct {
	Filename string `long:"filename" short:"f" description:"png filename" required:"true"`
}

func printUsageAndExit(exitcode int) {
	parser.WriteHelp(os.Stdout)
	fmt.Println()
	os.Exit(exitcode)

}

func printErrorAndExit(err error, exitcode int) {
	out := color.New(color.FgRed).Add(color.Underline)

	fmt.Println()
	if err != nil {
		out.Println(err.Error())
		fmt.Println()
	}
	printUsageAndExit(exitcode)
}

func scaleBarcode(code barcode.Barcode, width, height int) barcode.Barcode {
	if width == 0 {
		height = 24 * code.Bounds().Max.Y
	}
	if height == 0 {
		height = 24 * code.Bounds().Max.Y
	}

	barcode, err := barcode.Scale(code, width, height)
	if err != nil {
		printErrorAndExit(err, exitUsage)
	}

	return barcode
}

func main() {

	parser = flags.NewParser(&opts, flags.HelpFlag)
	//parser.Usage = "command -filename= [options] code"
	parser.ShortDescription = "barcode output"
	parser.LongDescription = "different barcodes can be created and saved as png."

	parser.AddCommand("ean",
		"EAN code",
		"the ean command creates a png with an ean barcode.",
		&eanCommand)

	parser.AddCommand("qr",
		"QR code",
		"the qr command creates a png with an QR code.",
		&qrCommand)

	parser.AddCommand("code128",
		"Code128 code",
		"the code128 command creates a png with an code128 code.",
		&code128Command)

	parser.AddCommand("pdf417",
		"PDF417 code",
		"the pdf417 command creates a png with an pdf417 code.",
		&pdf417Command)

	_, err := parser.Parse()
	if err != nil {

		switch err.(type) {
		case *flags.Error:

			e := err.(*flags.Error)
			if e.Type == flags.ErrHelp {
				printUsageAndExit(exitOK)
			} else {
				printErrorAndExit(err, exitUsage)
			}

		}

	}

	os.Exit(exitOK)

}
